import sqlalchemy
from sqlalchemy.orm import sessionmaker

class Connection:
    def __init__(self, **kwargs) -> None:
        self.user = kwargs.get('user')
        self.password = kwargs.get('password')
        self.database = kwargs.get('database')
        
        self.__engine = None
        self.session = None
        self.validate_and_connect()

    def validate_and_connect(self):
        if self.user is None or self.password is None or self.database is None:
            raise Exception('No valid database credentials specified')
        
        self.connect()

    def connect(self):
        try:
            if self.valid_parameters() is False:
                raise Exception('Invalid parameters')
            
            self.__engine = sqlalchemy.create_engine(
                f"mysql+pymysql://{self.user}:{self.password}@db/{self.database}"
            )

            self.session = sessionmaker(bind=self.__engine)
        except Exception as e:
            print(e)
    
    def valid_parameters(self) -> bool:
        if self.user is None or self.password is None or self.database is None:
            return False
        return True
 
    
    def set_user(self, user):
        self.user = user
    
    def get_user(self):
        return self.user

    def set_password(self, password):
        self.password = password
    
    def get_password(self):
        return self.password
    
    def get_engine(self):
        return self.__engine          
            




from sqlalchemy import Column, JSON, String, BigInteger, DateTime, MetaData, create_engine
from sqlalchemy.orm import declarative_base

db = create_engine('mysql+pymysql://root:secret@db/lucas')
metadata = MetaData(db)
Base = declarative_base(metadata=metadata)

class CallEventLog(Base):
    __tablename__ = 'call_event_logs'

    id = Column(BigInteger, primary_key=True)
    tenant_id = Column(String)
    account_sid = Column(String)
    call_sid = Column(String)
    status = Column(String)
    caller = Column(String)
    caller_country = Column(String)
    caller_state = Column(String)
    caller_city = Column(String)
    caller_zip = Column(String)
    callee = Column(String)
    callee_country = Column(String)
    callee_state = Column(String)
    callee_city = Column(String)
    callee_zip = Column(String)
    payload = Column(JSON)




from sqlalchemy import Column, JSON, String, BigInteger, DateTime, MetaData, create_engine, Boolean,FLOAT
from sqlalchemy.orm import declarative_base

db = create_engine('mysql+pymysql://root:secret@db/lucas')
metadata = MetaData(db)
Base = declarative_base(metadata=metadata)

class Call(Base):
    __tablename__ = 'calls'
    id = Column(BigInteger, primary_key=True)
    tenant_id = Column(String(70))
    account_sid = Column(String(70), nullable=True)
    sid = Column(String(70))
    user_id = Column(String(70),nullable=True)
    appointment_setter_id = Column(String(70),nullable=True)
    receiver_id = Column(String(70),nullable=True)
    opportunity_id = Column(BigInteger, nullable=True)
    recording_sid = Column(String(40), nullable=True)
    recording_duration = Column(FLOAT, default=0)
    conference_sid = Column(String(40), nullable=True)
    last_status = Column(String(20), nullable=True)
    direction = Column(String(20), nullable=True)
    from_ = Column(String(255), nullable=True)
    to = Column(String(255), nullable=True)
    start = Column(DateTime, nullable=True)
    end = Column(DateTime, nullable=True)
    task = Column(Boolean, default=False)
    task_sid = Column(String(40), nullable=True)
    start_leg = Column(Boolean, default=False)
    recorded = Column(Boolean, default=False)
    conference_moderator = Column(Boolean, default=False)
    conference_name = Column(String(40), nullable=True)
    listening = Column(Boolean, default=False)
    recording_url = Column(String(255), nullable=True)
    recording_migrated = Column(Boolean, default=False)
    transcribed = Column(Boolean, default=False)
    transcribing = Column(Boolean, default=False)
    transferred = Column(Boolean, default=False)
    called_state = Column(String(20), nullable=True)
    called_zip = Column(String(20), nullable=True)
    called_country = Column(String(20), nullable=True)
    called_city = Column(String(20), nullable=True)
    caller_id = Column(BigInteger, nullable=True)
    caller_state = Column(String(20), nullable=True)
    caller_zip = Column(String(20), nullable=True)
    caller_country = Column(String(20), nullable=True)
    caller_city = Column(String(20), nullable=True)
    coaching = Column(Boolean, default=False)
    payload = Column(JSON)
    metadata_ = Column(JSON)


import os, time, json
import threading

import pika
from pika.exceptions import AMQPConnectionError

service_name=os.environ["SERVICE_NAME"]
bus_events_exchange=os.environ["BUS_EVENTS_EXCHANGE"]
queue_name=f"service_{service_name}"

def callback(ch, method, properties, body):
    print(f"[{service_name}] Received %s" % body)
    ch.basic_ack(delivery_tag=method.delivery_tag)


class Rabbit:
    def __init__(self) -> None:
        self.debug = True
        self.host = "message_broker"
        self.connection = None
        self.channel = None
        self.queue = None
    
    def __enter__(self):
        self.open_connection()
        return self
    
    def __exit__(self, exc_type, exc_value, exc_tb):
        self.close_connection()
    
    def reset_values(self):
        self.connection = None
        self.channel = None
        self.queue = None
        print("values reseted")
    
    def log(self, *args):
        if(self.debug):
            print(args)
    
    def open_connection(self):
        if self.connection: 
            return self.connection
        self.connection = pika.BlockingConnection(pika.ConnectionParameters(host=self.host))
        return self

    def get_connection(self):
        return self.connection
    
    def open_channel(self):
        if self.channel:
            return self.channel
        self.channel = self.connection.channel()
        return self

    def get_channel(self):
        return self.channel
    
    def set_exchange(self, **kwargs):
        self.open_connection()
        self.open_channel()
        self.channel.exchange_declare(**kwargs)
        return self
    
    def new_queue(self,**kwargs):
        self.queue = self.channel.queue_declare(**kwargs)
        return self
    
    def get_queue(self):
        return self.queue

    def bind_queue(self,**kwargs):
        self.channel.queue_bind(**kwargs)
        return self

    def connect_to_bus_events(self, events_handler=None):
        try:
            type_of_exchange='fanout'
            self.set_exchange(exchange=bus_events_exchange,exchange_type=type_of_exchange)
            self.new_queue(queue=queue_name,durable=True)
            self.bind_queue(exchange=bus_events_exchange,queue=queue_name,routing_key='')
            self.connection.close()
            print(f"{service_name} connected to Bus Events 🚌")
            self.reset_values()
            self.start_consuming_from_bus_events(events_handler)
        except AMQPConnectionError:
            print("AMQPConnectionError Waiting for RabbitMQ server to be online...")
            time.sleep(3)
            self.connect_to_bus_events()
        except Exception as e:
            print (e)
            print("ErrorConnectingToBusEvents")

    def start_consuming_from_bus_events(self,events_handler=None):
        self.open_connection()
        self.open_channel()
        print(f"[{service_name}] Waiting for messages.")
        self.channel.basic_qos(prefetch_count=1)
        callback_function = callback if events_handler is None else events_handler.callback
        self.channel.basic_consume(queue=queue_name,on_message_callback=lambda ch, method, properties, body: callback_function(ch, method, properties, body))
        threading.Thread(target=lambda: self.channel.start_consuming()).start()
        # thread = threading.Thread(target=self.channel.start_consuming)
        # thread.start()
        # thread.join()

    def close_connection(self):
        if self.connection:
            self.connection.close()

    def fetch_new_event_payload(self):
        return {
            "event":None,
            "payload":{}
        }
    
    def put_on_bus_events(self,data):
        type_of_exchange='fanout'
        self.set_exchange(exchange=bus_events_exchange,exchange_type=type_of_exchange)
        self.channel.basic_publish(exchange=bus_events_exchange, routing_key='', body=json.dumps(data))
        print(f"Event {data['event']} published to bus events")

        
    

    




from sqlalchemy import Column, JSON, Boolean, String, BigInteger, Integer, DateTime, MetaData, create_engine
from sqlalchemy.orm import declarative_base

db = create_engine('mysql+pymysql://root:secret@db/lucas')
metadata = MetaData(db)
Base = declarative_base(metadata=metadata)

class ConferenceEventLog(Base):
    __tablename__ = 'conference_event_logs'

    id = Column(BigInteger, primary_key=True)
    tenant_id = Column(String)
    account_sid = Column(String)
    conference_sid = Column(String)
    call_sid=Column(String)
    friendly_name=Column(String)
    sequence_number = Column(Integer)
    status_callback_event = Column(String)
    payload = Column(JSON)




from setuptools import setup

with open('README.md', 'r') as puntero:
    long_description = puntero.read()


setup(
    name='lucas_helpers_webforce',
    packages=['lucas_helpers_webforce'],  # this must be the same as the name above
    version='0.4.3',
    description='description',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Webforce',
    author_email='',
    # use the URL to the github repo
    url='',
    download_url='',
    keywords=['testing', 'logging', 'example'],
    classifiers=[ ],
    license='MIT',
)